selfpace1.rank <- function(dev_decval, dev_labels, pos_lambda1_rank, neg_lambda1_rank) {
	#implement of the self-paced learning
	#1) the self-paced parameters (\lambda) are set by the sample rank as this strategy proves robust than setting by absolute values.
	#Given the \lambda rank we can easily calculate the \lambda reversely.
	#2) the positive and negative sample are paced seperately (using different parameters) as the data set is significantly unbalanced
	
	#calculate the loss
	loss = (dev_decval-dev_labels)^2	#squared error
	#loss = 1/(1+e^(-1*loss))			#logistic
	
	#self-paced learning for positive samples
	possidx = which(dev_labels==1)	#postive id mapping
	posloss = loss[possidx]
	possortedidx = sort(posloss, decreasing=FALSE, index.return=TRUE)$ix
	numtoselect = min(length(possortedidx), pos_lambda1_rank)
	selectedposidx = possidx[possortedidx[1:numtoselect]]
	
	#self-paced learning for negative samples
	negidx = which(dev_labels==0)	#negative id mapping
	negloss = loss[negidx]
	negsortedidx = sort(negloss, decreasing=FALSE, index.return=TRUE)$ix
	numtoselect = min(length(negsortedidx), neg_lambda1_rank)
	selectednegidx = negidx[negsortedidx[1:numtoselect]]

	#return the result
	result = list(selectedposidx, selectednegidx)
	names(result) <- c("pos", "neg")
	return(result)
}


findbestiter1 <- function(devaps) {
	#find the best iteration
	bestidx = replicate(ncol(devaps),0)
	for(eventid in 1:ncol(devaps)) {
		tempidx = which(devaps[,eventid]==max(devaps[,eventid]))
		#if there is a tie, randomly pick one.
		bestidx[eventid] = sample(tempidx,1)
	}
	return(bestidx)
}

findbestiter2 <- function(devaps, favor_bigger_iter=TRUE) {
	#find the best iteration
	bestidx = replicate(ncol(devaps),0)
	for(eventid in 1:ncol(devaps)) {
		tempidx = which(devaps[,eventid]==max(devaps[,eventid]))
		#if there is a tie, use the one in the biggest or smallest iteration.
		if(favor_bigger_iter) bestidx[eventid] = tempidx[length(tempidx)]
		else bestidx[eventid] = tempidx[1]
	}
	return(bestidx)
}

selfpace2.rank <- function(dev_decval, dev_labels, dev_membership, pos_lambda1_rank, neg_lambda1_rank, diversity_ratio = 0.01) {
	#implement of the self-paced learning with diversity
	#like in self-paced learning implementations
	#1) the self-paced parameters (\lambda) are set by the sample rank as this strategy proves robust than setting by absolute values.
	#Given the \lambda rank we can easily reversely derive the \lambda.
	#2) the positive and negative sample are paced seperately (using different parameters) as the data set is significantly unbalanced
	#3) since the parameters are set by rank, the diversity parameter \gamma is set by diversity ratio which balances the easiness and 	
	#diversity. Given the rank parameters of \lambda and diversity_ratio, we can reversely derive the \lambda and \gamma. 
	
	#calculate the loss
	loss = (dev_decval-dev_labels)^2	#squared error
	numgourps = max(dev_membership)		#number of goups
	
	#this 2-column matrix store the loss and the threshold for each sample
	grouploss_threshold = matrix(0, nrow = length(dev_labels), ncol = 2)
	
	#calculate the threshold for each sample according to Algorithm 1 in paper: Self-paced Learning with Diversity.
	for(i in 1:numgourps) {	#for each group
		thisgroupidx = which(dev_membership==i)
		thisgroupedloss = loss[thisgroupidx]	#loss of the samples in the group
		thisgroupedrank = rank(thisgroupedloss)	#rank of that sample in the group
		thisgroupedthreshold = 1/(sqrt(thisgroupedrank) + sqrt(thisgroupedrank-1))	#rank of that sample in the group
		grouploss_threshold[thisgroupidx,] = cbind(thisgroupedloss, thisgroupedthreshold)
	}
	
	posidx = which(dev_labels==1)	#postive id mapping
	negidx = which(dev_labels==0)	#negative id mapping
	
	#calculate pos_lambda1 and neg_lambda2 according to the rank
	#pos_lambda1 = sort(grouploss_threshold[posidx,1])[min(length(posidx), pos_lambda1_rank)]
	#pos_lambda2 = sort(grouploss_threshold[negidx,1])[min(length(negidx), neg_lambda1_rank)]
	
	
	#metric = loss - \gamma \frac{1}{\sqrt{i}+\sqrt{i-1}}
	metric = grouploss_threshold[,1] - diversity_ratio * grouploss_threshold[,2]
	
	#rank every sample by the metric and selected top lambda_1_rank
	#it is like first sorting sampled based on the metric and then select top lambda1_rank
	selectedposidx = posidx[which(rank(metric[posidx]) <= min(length(posidx), pos_lambda1_rank))]
	selectednegidx = negidx[which(rank(metric[negidx]) <= min(length(negidx), neg_lambda1_rank))]
	
	result = list(selectedposidx, selectednegidx)
	names(result) <- c("pos", "neg")
	return(result)
}


plotit <- function(input, outfilename = NULL) {
	if(!is.null(outfilename)) {
		win.metafile(outfilename)
		#postscript(outfilename, width = 6.0, height = 4.0)	#save it in eps
	}
	curve = input$devevl
	x_range = 1:nrow(curve)
	y_range <- range(curve[,1], curve[,2])
	
	plot(curve[,1], type="o", col="blue", ylim=y_range, ann=FALSE)
	title(xlab="Iteration", ylab="Mean Average Precision", cex.lab = 1.2)
	#curve[,1] = curve[,1]/sum(curve[,1])
	#curve[,2] = curve[,2]/sum(curve[,2])
	
	lines(x_range, curve[,1], col = "blue", type="o")
	lines(x_range, curve[,2], col = "red", pch=22, lty=1, type="o")
	legend("bottomright", c("Dev MAP","Test MAP"), cex=1.2, col=c("blue","red"), pch=21:22, lty=1:2)
	
	if(!is.null(outfilename)) dev.off()
	
	
	if(!is.null(outfilename)) {
		png(paste(outfilename,".png", sep=""))
	}
	curve = input$devevl
	x_range = 1:nrow(curve)
	y_range <- range(curve[,1], curve[,2])
	
	plot(curve[,1], type="o", col="blue", ylim=y_range, ann=FALSE)
	title(xlab="Iteration", ylab="Mean Average Precision", cex.lab = 1.2)
	#curve[,1] = curve[,1]/sum(curve[,1])
	#curve[,2] = curve[,2]/sum(curve[,2])
	
	lines(x_range, curve[,1], col = "blue", type="o")
	lines(x_range, curve[,2], col = "red", pch=22, lty=1, type="o")
	legend("bottomright", c("Dev MAP","Test MAP"), cex=1.2, col=c("blue","red"), pch=21:22, lty=1:2)
	
	if(!is.null(outfilename)) dev.off()
	
	write(t(input$devevl), file = paste(outfilename,".txt", sep=""), ncolumns = ncol(input$devevl))
}
